/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            renderdialog.h
 *
 *  Thu Sep 20 17:32:11 CEST 2018
 *  Copyright 2018 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of DrumGizmo.
 *
 *  DrumGizmo is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DrumGizmo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DrumGizmo; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#pragma once

#include <QDialog>
#include <QProgressBar>
#include <QTreeView>

#include "projectrenderer.h"

class Project;
class RenderDataModel;
class ProgressDelegate;

class RenderDialog
	: public QDialog
{
	Q_OBJECT
public:
	RenderDialog(QWidget* parent, Project& project);

private slots:
	void render();

	void progressStart(int total);
	void progressTask(QString text);

	void progressRenderStart(int tasktotal);
	void progressRenderTask(int subtask);
	void progressRenderFinished(int success);

	void progressFinished(int success);

	void chooseExportPath();
	void exportPathChanged(const QString&);

private:
	Project& project;
	ProjectRenderer renderer;

	QLineEdit* export_path;
	QPushButton* export_btn;
	QProgressBar* bar;

	QTreeView* tasks;

	RenderDataModel* model;
	ProgressDelegate* delegate;
};
