/****************************************************************************
** Meta object code from reading C++ file 'samplesorter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "samplesorter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'samplesorter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SampleSorter_t {
    QByteArrayData data[19];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SampleSorter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SampleSorter_t qt_meta_stringdata_SampleSorter = {
    {
QT_MOC_LITERAL(0, 0, 12), // "SampleSorter"
QT_MOC_LITERAL(1, 13, 10), // "setWavData"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 12), // "const float*"
QT_MOC_LITERAL(4, 38, 4), // "data"
QT_MOC_LITERAL(5, 43, 6), // "size_t"
QT_MOC_LITERAL(6, 50, 4), // "size"
QT_MOC_LITERAL(7, 55, 15), // "setAttackLength"
QT_MOC_LITERAL(8, 71, 3), // "len"
QT_MOC_LITERAL(9, 75, 12), // "attackLength"
QT_MOC_LITERAL(10, 88, 12), // "addSelection"
QT_MOC_LITERAL(11, 101, 8), // "sel_id_t"
QT_MOC_LITERAL(12, 110, 2), // "id"
QT_MOC_LITERAL(13, 113, 19), // "addSelectionPreview"
QT_MOC_LITERAL(14, 133, 8), // "relayout"
QT_MOC_LITERAL(15, 142, 14), // "setShowPreview"
QT_MOC_LITERAL(16, 157, 12), // "show_preview"
QT_MOC_LITERAL(17, 170, 15), // "setSpreadFactor"
QT_MOC_LITERAL(18, 186, 6) // "spread"

    },
    "SampleSorter\0setWavData\0\0const float*\0"
    "data\0size_t\0size\0setAttackLength\0len\0"
    "attackLength\0addSelection\0sel_id_t\0"
    "id\0addSelectionPreview\0relayout\0"
    "setShowPreview\0show_preview\0setSpreadFactor\0"
    "spread"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SampleSorter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x0a /* Public */,
       7,    1,   59,    2, 0x0a /* Public */,
       9,    0,   62,    2, 0x0a /* Public */,
      10,    1,   63,    2, 0x0a /* Public */,
      13,    1,   66,    2, 0x0a /* Public */,
      14,    0,   69,    2, 0x0a /* Public */,
      15,    1,   70,    2, 0x0a /* Public */,
      17,    1,   73,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Int,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Int,   18,

       0        // eod
};

void SampleSorter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SampleSorter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setWavData((*reinterpret_cast< const float*(*)>(_a[1])),(*reinterpret_cast< size_t(*)>(_a[2]))); break;
        case 1: _t->setAttackLength((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: { int _r = _t->attackLength();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->addSelection((*reinterpret_cast< sel_id_t(*)>(_a[1]))); break;
        case 4: _t->addSelectionPreview((*reinterpret_cast< sel_id_t(*)>(_a[1]))); break;
        case 5: _t->relayout(); break;
        case 6: _t->setShowPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->setSpreadFactor((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SampleSorter::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SampleSorter.data,
    qt_meta_data_SampleSorter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SampleSorter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SampleSorter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SampleSorter.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SampleSorter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
