/****************************************************************************
** Meta object code from reading C++ file 'renderdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "renderdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'renderdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RenderDialog_t {
    QByteArrayData data[16];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RenderDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RenderDialog_t qt_meta_stringdata_RenderDialog = {
    {
QT_MOC_LITERAL(0, 0, 12), // "RenderDialog"
QT_MOC_LITERAL(1, 13, 6), // "render"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 13), // "progressStart"
QT_MOC_LITERAL(4, 35, 5), // "total"
QT_MOC_LITERAL(5, 41, 12), // "progressTask"
QT_MOC_LITERAL(6, 54, 4), // "text"
QT_MOC_LITERAL(7, 59, 19), // "progressRenderStart"
QT_MOC_LITERAL(8, 79, 9), // "tasktotal"
QT_MOC_LITERAL(9, 89, 18), // "progressRenderTask"
QT_MOC_LITERAL(10, 108, 7), // "subtask"
QT_MOC_LITERAL(11, 116, 22), // "progressRenderFinished"
QT_MOC_LITERAL(12, 139, 7), // "success"
QT_MOC_LITERAL(13, 147, 16), // "progressFinished"
QT_MOC_LITERAL(14, 164, 16), // "chooseExportPath"
QT_MOC_LITERAL(15, 181, 17) // "exportPathChanged"

    },
    "RenderDialog\0render\0\0progressStart\0"
    "total\0progressTask\0text\0progressRenderStart\0"
    "tasktotal\0progressRenderTask\0subtask\0"
    "progressRenderFinished\0success\0"
    "progressFinished\0chooseExportPath\0"
    "exportPathChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RenderDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x08 /* Private */,
       3,    1,   60,    2, 0x08 /* Private */,
       5,    1,   63,    2, 0x08 /* Private */,
       7,    1,   66,    2, 0x08 /* Private */,
       9,    1,   69,    2, 0x08 /* Private */,
      11,    1,   72,    2, 0x08 /* Private */,
      13,    1,   75,    2, 0x08 /* Private */,
      14,    0,   78,    2, 0x08 /* Private */,
      15,    1,   79,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void RenderDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RenderDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->render(); break;
        case 1: _t->progressStart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->progressTask((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->progressRenderStart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->progressRenderTask((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->progressRenderFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->progressFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->chooseExportPath(); break;
        case 8: _t->exportPathChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RenderDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_RenderDialog.data,
    qt_meta_data_RenderDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RenderDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RenderDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RenderDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int RenderDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
