/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            localehandler.cc
 *
 *  Sat May  5 12:39:49 CEST 2018
 *  Copyright 2018 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of DrumGizmo.
 *
 *  DrumGizmo is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DrumGizmo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DrumGizmo; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "localehandler.h"

#include <QApplication>
#include <QLocale>
#include <QTranslator>
#include <QString>
#include <QFile>
#include <QDir>

LocaleHandler::LocaleHandler(QApplication& app)
{
	QString locale = QLocale().name().section('_', 0, 0);
	if(!locale.isEmpty() && locale != "C")
	{
		QString file = QString(LOCALEDIR) + QDir::separator() + "dgedit_" + locale + ".qm";
		if(QFile::exists(file))
		{
			translator.load(file);
			app.installTranslator(&translator);
		}
	}
}
