/****************************************************************************
** Meta object code from reading C++ file 'instrumentwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "instrumentwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'instrumentwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_InstrumentWidget_t {
    QByteArrayData data[13];
    char stringdata0[153];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_InstrumentWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_InstrumentWidget_t qt_meta_stringdata_InstrumentWidget = {
    {
QT_MOC_LITERAL(0, 0, 16), // "InstrumentWidget"
QT_MOC_LITERAL(1, 17, 8), // "doExport"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 8), // "loadFile"
QT_MOC_LITERAL(4, 36, 8), // "filename"
QT_MOC_LITERAL(5, 45, 11), // "playSamples"
QT_MOC_LITERAL(6, 57, 6), // "browse"
QT_MOC_LITERAL(7, 64, 10), // "tabChanged"
QT_MOC_LITERAL(8, 75, 5), // "tabid"
QT_MOC_LITERAL(9, 81, 22), // "generateSlidersChanged"
QT_MOC_LITERAL(10, 104, 16), // "selectionChanged"
QT_MOC_LITERAL(11, 121, 13), // "prefixChanged"
QT_MOC_LITERAL(12, 135, 17) // "exportPathChanged"

    },
    "InstrumentWidget\0doExport\0\0loadFile\0"
    "filename\0playSamples\0browse\0tabChanged\0"
    "tabid\0generateSlidersChanged\0"
    "selectionChanged\0prefixChanged\0"
    "exportPathChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_InstrumentWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x0a /* Public */,
       3,    1,   60,    2, 0x0a /* Public */,
       5,    0,   63,    2, 0x0a /* Public */,
       6,    0,   64,    2, 0x0a /* Public */,
       7,    1,   65,    2, 0x0a /* Public */,
       9,    0,   68,    2, 0x0a /* Public */,
      10,    0,   69,    2, 0x0a /* Public */,
      11,    0,   70,    2, 0x0a /* Public */,
      12,    0,   71,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void InstrumentWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<InstrumentWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->doExport(); break;
        case 1: _t->loadFile((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->playSamples(); break;
        case 3: _t->browse(); break;
        case 4: _t->tabChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->generateSlidersChanged(); break;
        case 6: _t->selectionChanged(); break;
        case 7: _t->prefixChanged(); break;
        case 8: _t->exportPathChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject InstrumentWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_InstrumentWidget.data,
    qt_meta_data_InstrumentWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *InstrumentWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InstrumentWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_InstrumentWidget.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int InstrumentWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
