/****************************************************************************
** Meta object code from reading C++ file 'canvastoolselections.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "canvastoolselections.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvastoolselections.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CanvasToolSelections_t {
    QByteArrayData data[15];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CanvasToolSelections_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CanvasToolSelections_t qt_meta_stringdata_CanvasToolSelections = {
    {
QT_MOC_LITERAL(0, 0, 20), // "CanvasToolSelections"
QT_MOC_LITERAL(1, 21, 20), // "autoCreateSelections"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 27), // "autoCreateSelectionsPreview"
QT_MOC_LITERAL(4, 71, 15), // "clearSelections"
QT_MOC_LITERAL(5, 87, 16), // "thresholdChanged"
QT_MOC_LITERAL(6, 104, 9), // "threshold"
QT_MOC_LITERAL(7, 114, 17), // "noiseFloorChanged"
QT_MOC_LITERAL(8, 132, 1), // "t"
QT_MOC_LITERAL(9, 134, 11), // "holdChanged"
QT_MOC_LITERAL(10, 146, 1), // "h"
QT_MOC_LITERAL(11, 148, 14), // "fadeoutChanged"
QT_MOC_LITERAL(12, 163, 1), // "f"
QT_MOC_LITERAL(13, 165, 14), // "setShowPreview"
QT_MOC_LITERAL(14, 180, 12) // "show_preview"

    },
    "CanvasToolSelections\0autoCreateSelections\0"
    "\0autoCreateSelectionsPreview\0"
    "clearSelections\0thresholdChanged\0"
    "threshold\0noiseFloorChanged\0t\0holdChanged\0"
    "h\0fadeoutChanged\0f\0setShowPreview\0"
    "show_preview"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CanvasToolSelections[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x0a /* Public */,
       3,    0,   55,    2, 0x0a /* Public */,
       4,    0,   56,    2, 0x0a /* Public */,
       5,    1,   57,    2, 0x0a /* Public */,
       7,    1,   60,    2, 0x0a /* Public */,
       9,    1,   63,    2, 0x0a /* Public */,
      11,    1,   66,    2, 0x0a /* Public */,
      13,    1,   69,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    6,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Bool,   14,

       0        // eod
};

void CanvasToolSelections::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CanvasToolSelections *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->autoCreateSelections(); break;
        case 1: _t->autoCreateSelectionsPreview(); break;
        case 2: _t->clearSelections(); break;
        case 3: _t->thresholdChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->noiseFloorChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->holdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->fadeoutChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->setShowPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CanvasToolSelections::staticMetaObject = { {
    QMetaObject::SuperData::link<CanvasTool::staticMetaObject>(),
    qt_meta_stringdata_CanvasToolSelections.data,
    qt_meta_data_CanvasToolSelections,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CanvasToolSelections::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CanvasToolSelections::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CanvasToolSelections.stringdata0))
        return static_cast<void*>(this);
    return CanvasTool::qt_metacast(_clname);
}

int CanvasToolSelections::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CanvasTool::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
