/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[24];
    char stringdata0[288];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 13), // "addInstrument"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 14), // "editInstrument"
QT_MOC_LITERAL(4, 41, 16), // "removeInstrument"
QT_MOC_LITERAL(5, 58, 23), // "instrumentDoubleClicked"
QT_MOC_LITERAL(6, 82, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(7, 99, 4), // "item"
QT_MOC_LITERAL(8, 104, 10), // "addChannel"
QT_MOC_LITERAL(9, 115, 13), // "removeChannel"
QT_MOC_LITERAL(10, 129, 20), // "channelDoubleClicked"
QT_MOC_LITERAL(11, 150, 17), // "updateWindowTitle"
QT_MOC_LITERAL(12, 168, 10), // "newProject"
QT_MOC_LITERAL(13, 179, 11), // "loadProject"
QT_MOC_LITERAL(14, 191, 8), // "filename"
QT_MOC_LITERAL(15, 200, 11), // "saveProject"
QT_MOC_LITERAL(16, 212, 13), // "saveProjectAs"
QT_MOC_LITERAL(17, 226, 14), // "projectChanged"
QT_MOC_LITERAL(18, 241, 8), // "closeTab"
QT_MOC_LITERAL(19, 250, 3), // "tab"
QT_MOC_LITERAL(20, 254, 4), // "test"
QT_MOC_LITERAL(21, 259, 6), // "render"
QT_MOC_LITERAL(22, 266, 11), // "editProject"
QT_MOC_LITERAL(23, 278, 9) // "showAbout"

    },
    "MainWindow\0addInstrument\0\0editInstrument\0"
    "removeInstrument\0instrumentDoubleClicked\0"
    "QListWidgetItem*\0item\0addChannel\0"
    "removeChannel\0channelDoubleClicked\0"
    "updateWindowTitle\0newProject\0loadProject\0"
    "filename\0saveProject\0saveProjectAs\0"
    "projectChanged\0closeTab\0tab\0test\0"
    "render\0editProject\0showAbout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x0a /* Public */,
       3,    0,  110,    2, 0x0a /* Public */,
       4,    0,  111,    2, 0x0a /* Public */,
       5,    1,  112,    2, 0x0a /* Public */,
       8,    0,  115,    2, 0x0a /* Public */,
       9,    0,  116,    2, 0x0a /* Public */,
      10,    1,  117,    2, 0x0a /* Public */,
      11,    0,  120,    2, 0x0a /* Public */,
      12,    0,  121,    2, 0x0a /* Public */,
      13,    0,  122,    2, 0x0a /* Public */,
      13,    1,  123,    2, 0x0a /* Public */,
      15,    0,  126,    2, 0x0a /* Public */,
      16,    0,  127,    2, 0x0a /* Public */,
      17,    0,  128,    2, 0x0a /* Public */,
      18,    1,  129,    2, 0x0a /* Public */,
      20,    0,  132,    2, 0x08 /* Private */,
      21,    0,  133,    2, 0x08 /* Private */,
      22,    0,  134,    2, 0x08 /* Private */,
      23,    0,  135,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->addInstrument(); break;
        case 1: _t->editInstrument(); break;
        case 2: _t->removeInstrument(); break;
        case 3: _t->instrumentDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 4: _t->addChannel(); break;
        case 5: _t->removeChannel(); break;
        case 6: _t->channelDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 7: _t->updateWindowTitle(); break;
        case 8: _t->newProject(); break;
        case 9: _t->loadProject(); break;
        case 10: _t->loadProject((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 11: _t->saveProject(); break;
        case 12: _t->saveProjectAs(); break;
        case 13: _t->projectChanged(); break;
        case 14: _t->closeTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->test(); break;
        case 16: _t->render(); break;
        case 17: _t->editProject(); break;
        case 18: _t->showAbout(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
