/****************************************************************************
** Meta object code from reading C++ file 'player.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "player.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'player.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Player_t {
    QByteArrayData data[26];
    char stringdata0[256];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Player_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Player_t qt_meta_stringdata_Player = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Player"
QT_MOC_LITERAL(1, 7, 10), // "peakUpdate"
QT_MOC_LITERAL(2, 18, 0), // ""
QT_MOC_LITERAL(3, 19, 4), // "peak"
QT_MOC_LITERAL(4, 24, 12), // "peakUpdateDB"
QT_MOC_LITERAL(5, 37, 7), // "peak_db"
QT_MOC_LITERAL(6, 45, 14), // "positionUpdate"
QT_MOC_LITERAL(7, 60, 6), // "size_t"
QT_MOC_LITERAL(8, 67, 8), // "position"
QT_MOC_LITERAL(9, 76, 10), // "setPcmData"
QT_MOC_LITERAL(10, 87, 6), // "float*"
QT_MOC_LITERAL(11, 94, 4), // "data"
QT_MOC_LITERAL(12, 99, 11), // "num_samples"
QT_MOC_LITERAL(13, 111, 10), // "samplerate"
QT_MOC_LITERAL(14, 122, 13), // "setGainScalar"
QT_MOC_LITERAL(15, 136, 11), // "gain_scalar"
QT_MOC_LITERAL(16, 148, 9), // "setGainDB"
QT_MOC_LITERAL(17, 158, 7), // "gain_db"
QT_MOC_LITERAL(18, 166, 11), // "setPosition"
QT_MOC_LITERAL(19, 178, 13), // "playSelection"
QT_MOC_LITERAL(20, 192, 9), // "Selection"
QT_MOC_LITERAL(21, 202, 9), // "selection"
QT_MOC_LITERAL(22, 212, 6), // "length"
QT_MOC_LITERAL(23, 219, 17), // "playSelectionDone"
QT_MOC_LITERAL(24, 237, 4), // "stop"
QT_MOC_LITERAL(25, 242, 13) // "reportTimeout"

    },
    "Player\0peakUpdate\0\0peak\0peakUpdateDB\0"
    "peak_db\0positionUpdate\0size_t\0position\0"
    "setPcmData\0float*\0data\0num_samples\0"
    "samplerate\0setGainScalar\0gain_scalar\0"
    "setGainDB\0gain_db\0setPosition\0"
    "playSelection\0Selection\0selection\0"
    "length\0playSelectionDone\0stop\0"
    "reportTimeout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Player[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,
       6,    1,   80,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    3,   83,    2, 0x0a /* Public */,
      14,    1,   90,    2, 0x0a /* Public */,
      16,    1,   93,    2, 0x0a /* Public */,
      18,    1,   96,    2, 0x0a /* Public */,
      19,    2,   99,    2, 0x0a /* Public */,
      19,    1,  104,    2, 0x2a /* Public | MethodCloned */,
      23,    0,  107,    2, 0x0a /* Public */,
      24,    0,  108,    2, 0x0a /* Public */,
      25,    0,  109,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    5,
    QMetaType::Void, 0x80000000 | 7,    8,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 7, QMetaType::Double,   11,   12,   13,
    QMetaType::Void, QMetaType::Double,   15,
    QMetaType::Void, QMetaType::Double,   17,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 20, QMetaType::Int,   21,   22,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Player::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Player *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->peakUpdate((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->peakUpdateDB((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->positionUpdate((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 3: _t->setPcmData((*reinterpret_cast< float*(*)>(_a[1])),(*reinterpret_cast< size_t(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 4: _t->setGainScalar((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 5: _t->setGainDB((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 6: _t->setPosition((*reinterpret_cast< size_t(*)>(_a[1]))); break;
        case 7: _t->playSelection((*reinterpret_cast< Selection(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->playSelection((*reinterpret_cast< Selection(*)>(_a[1]))); break;
        case 9: { bool _r = _t->playSelectionDone();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->stop(); break;
        case 11: _t->reportTimeout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Player::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Player::peakUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Player::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Player::peakUpdateDB)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Player::*)(size_t );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Player::positionUpdate)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Player::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_Player.data,
    qt_meta_data_Player,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Player::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Player::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Player.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int Player::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void Player::peakUpdate(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Player::peakUpdateDB(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Player::positionUpdate(size_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
