/****************************************************************************
** Meta object code from reading C++ file 'filelist.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "filelist.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filelist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileList_t {
    QByteArrayData data[17];
    char stringdata0[181];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileList_t qt_meta_stringdata_FileList = {
    {
QT_MOC_LITERAL(0, 0, 8), // "FileList"
QT_MOC_LITERAL(1, 9, 17), // "masterFileChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 8), // "filename"
QT_MOC_LITERAL(4, 37, 11), // "fileRemoved"
QT_MOC_LITERAL(5, 49, 4), // "file"
QT_MOC_LITERAL(6, 54, 4), // "name"
QT_MOC_LITERAL(7, 59, 15), // "allFilesRemoved"
QT_MOC_LITERAL(8, 75, 8), // "addFiles"
QT_MOC_LITERAL(9, 84, 16), // "selectionChanged"
QT_MOC_LITERAL(10, 101, 11), // "QModelIndex"
QT_MOC_LITERAL(11, 113, 5), // "index"
QT_MOC_LITERAL(12, 119, 19), // "onCustomContextMenu"
QT_MOC_LITERAL(13, 139, 5), // "point"
QT_MOC_LITERAL(14, 145, 9), // "setMaster"
QT_MOC_LITERAL(15, 155, 10), // "removeFile"
QT_MOC_LITERAL(16, 166, 14) // "removeAllFiles"

    },
    "FileList\0masterFileChanged\0\0filename\0"
    "fileRemoved\0file\0name\0allFilesRemoved\0"
    "addFiles\0selectionChanged\0QModelIndex\0"
    "index\0onCustomContextMenu\0point\0"
    "setMaster\0removeFile\0removeAllFiles"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    2,   62,    2, 0x06 /* Public */,
       7,    0,   67,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   68,    2, 0x0a /* Public */,
       9,    1,   69,    2, 0x08 /* Private */,
      12,    1,   72,    2, 0x08 /* Private */,
      14,    0,   75,    2, 0x08 /* Private */,
      15,    0,   76,    2, 0x08 /* Private */,
      16,    0,   77,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,    6,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QPoint,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void FileList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->masterFileChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->fileRemoved((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 2: _t->allFilesRemoved(); break;
        case 3: _t->addFiles(); break;
        case 4: _t->selectionChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 5: _t->onCustomContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 6: _t->setMaster(); break;
        case 7: _t->removeFile(); break;
        case 8: _t->removeAllFiles(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileList::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileList::masterFileChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileList::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileList::fileRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileList::allFilesRemoved)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileList::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_meta_stringdata_FileList.data,
    qt_meta_data_FileList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileList.stringdata0))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int FileList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void FileList::masterFileChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FileList::fileRemoved(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FileList::allFilesRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
